import logging
from typing import Optional

from slack_sdk import WebClient
from slack_sdk.errors import SlackApiError
from slack_sdk.web.slack_response import SlackResponse

from hartware_lib.exceptions.slack import ApiError

logger = logging.getLogger(__name__)


class SlackAdapter:
    def __init__(self, config):
        self.config = config
        self.client = WebClient(token=self.config.api_token)

    def send(self, msg: str, channel: Optional[str] = None) -> SlackResponse:
        if not channel and self.config.default_channel:
            channel = self.config.default_channel

        try:
            return self.client.chat_postMessage(
                channel=channel or self.config.default_channel, text=msg
            )
        except SlackApiError:
            api_error = ApiError()
            logger.warning(api_error.message)
            raise api_error



from dataclasses import dataclass
from os import environ


@dataclass
class AppConfig:
    TEST: bool = environ.get("TEST", "").lower() in ("1", "y", "yes", "true")
    DEBUG: bool = environ.get("DEBUG", "").lower() in ("1", "y", "yes", "true")



import argparse

from hartware_lib.adapters.slack import SlackAdapter
from hartware_lib.core.settings.slack import SlackConfig


def main():
    parser = argparse.ArgumentParser(description="Slack message sender.")
    parser.add_argument("message", help="set the message")
    parser.add_argument("-c", "--channel", help="set the channel")

    args = parser.parse_args()

    SlackAdapter(SlackConfig()).send(
        args.message,
        channel=args.channel,
    )



from typing import Optional

from pydantic import BaseModel, BaseSettings, Field


class SlackConfig(BaseModel):
    api_token: str
    default_channel: str


class SlackEnvConfig(BaseSettings):
    api_token: Optional[str] = Field(env="SLACK_BOT_TOKEN")
    default_channel: Optional[str] = Field(env="SLACK_BOT_DEFAULT_CHANNEL")
